;;; a tabbed-radio-layout-pane using presentations
;;;
;;; date: Sat, 17 Sep 2005 23:52:15 +0200
;;; Max-Gerd Retzlaff <m.retzlaff@gmx.net>

(in-package :clim-user)


(defclass tabbed-layout-pane (vrack-pane)
  ((tabbed-panes :initform nil :accessor tabbed-panes-of-tabbed-layout :initarg :tabbed-panes)
   (radio-layout-pane :initform nil :accessor radio-layout-pane-of-tabbed-layout :initarg :radio-layout-pane)
   (tab-bar-pane :initform nil :accessor tab-bar-pane-of-tabbed-layout :initarg :tab-bar-pane)))

(defclass tabbed-pane ()
  ((title :initform nil :accessor tabbed-pane-title :initarg :title)
   (pane :initform nil :accessor tabbed-pane-pane :initarg :pane)
   (ptype :initform nil :accessor tabbed-pane-ptype :initarg :ptype)
   (drawing-options :initform nil :accessor drawing-options-of-tabbed-pane)))

(defgeneric find-in-tabbed-panes-list (thing parent))

(defmethod find-in-tabbed-panes-list ((thing sheet) (parent tabbed-layout-pane))
  (find thing (tabbed-panes-of-tabbed-layout parent)
        :key #'tabbed-pane-pane :test #'equal))

(defmethod find-in-tabbed-panes-list ((thing string) (parent tabbed-layout-pane))
  (find thing (tabbed-panes-of-tabbed-layout parent)
        :key #'tabbed-pane-title :test #'string-equal))

(defmethod find-in-tabbed-panes-list ((thing sheet) (parent (eql 'tabbed-layout-pane)))
  (find-in-tabbed-panes-list thing (sheet-parent (sheet-parent thing))))

(defun set-drawing-options-for-pane-in-tabbed-layout (pane drawing-options)
  (setf (drawing-options-of-tabbed-pane (find-in-tabbed-panes-list pane 'tabbed-layout-pane))
        drawing-options))

(defgeneric enabled-pane (parent))

(defmethod enabled-pane ((parent tabbed-layout-pane))
  (find-in-tabbed-panes-list
   (enabled-pane (radio-layout-pane-of-tabbed-layout parent))
   parent))

(defun make-tabbed-pane-from-list (title tabbed-pane &optional ptype)
  (make-instance 'tabbed-pane
                 :title title
                 :pane tabbed-pane
                 :ptype ptype))

(defun make-list-from-tabbed-pane (tabbed-pane)
  (list (tabbed-pane-title tabbed-pane)
        (tabbed-pane-pane tabbed-pane)
        (tabbed-pane-ptype tabbed-pane)))



(define-command (com-switch-to-tab :command-table clim:global-command-table)
    ((tabbed-pane 'tabbed-pane :prompt "Tabbed pane"))
  (let ((pane (tabbed-pane-pane tabbed-pane)))
    (switch-to-pane pane 'radio-layout-pane)))

(define-presentation-to-command-translator switch-via-tab-button
    (tabbed-pane com-switch-to-tab clim:global-command-table
                 :gesture :select
                 :documentation "Switch to this pane"
                 :pointer-documentation "Switch to this pane")
    (object)
  (list object))

(define-command (com-remove-tabbed-pane :command-table clim:global-command-table)
    ((tabbed-pane 'tabbed-pane :prompt "Tabbed pane"))
  (remove-pane tabbed-pane 'tabbed-layout-pane))

;;;You probably don't want to uncomment this general command translator.
;;;
;;; (define-presentation-to-command-translator remove-tabbed-pane
;;;     (tabbed-pane com-remove-tabbed-pane clim:global-command-table
;;;                  :gesture :describe
;;;                  :documentation "remove this tabbed pane")
;;;     (object)
;;;   (list object))

;;; a test command
(define-command (com-narf :name t
                          :command-table clim:global-command-table)
    ((pane 'pane :prompt "Tabbed pane"))
  (print (pane-name pane))
  (set-drawing-options-for-pane-in-tabbed-layout pane (list :ink +red+))
  #+nil (break))



(defclass tab-bar-view (gadget-view)
  ())

(defparameter +tab-bar-view+ (make-instance 'tab-bar-view))

(define-presentation-method present (tabbed-pane (type tabbed-pane) stream
                                                (view tab-bar-view) &key)
  (stream-increment-cursor-position stream 5 0)
  (multiple-value-bind (x y) (stream-cursor-position stream)
    (let* ((length-top-line (+ x 6 (text-size stream (tabbed-pane-title tabbed-pane)) 3))
           (tab-button-polygon (list x (+ y 14)   (+ x 6) y
                                    (+ x 6) y   length-top-line y
                                    length-top-line y   (+ length-top-line 6) (+ y 14))))
      
      ;; grey-filled polygone for the disabled panes
      (unless (sheet-enabled-p (tabbed-pane-pane tabbed-pane))
        (draw-polygon* stream tab-button-polygon :ink +grey+))
      
      ;; black non-filled polygon
      (draw-polygon* stream tab-button-polygon :ink +black+ :filled nil)
      
      ;; "breach" the underline for the enabled pane
      (when (sheet-enabled-p (tabbed-pane-pane tabbed-pane))
        ;; (draw-line* stream x (+ y 14) (+ length-top-line 6) (+ y 14) :ink +white+)
        (draw-line stream (apply #'make-point (subseq tab-button-polygon 0 2))
                          (apply #'make-point (subseq tab-button-polygon (- (length tab-button-polygon) 2)))
                          :ink +background-ink+))))
  
  (stream-increment-cursor-position stream 8 0)
  (apply #'invoke-with-drawing-options stream
         (lambda (rest)
           (declare (ignore rest))
           (write-string (tabbed-pane-title tabbed-pane) stream))
         (drawing-options-of-tabbed-pane tabbed-pane))
  (stream-increment-cursor-position stream 10 0))



(defmacro with-tabbed-layout ((default-ptype &key name)
                              &body body)
  (let* ((radio-layout-pane (gensym "radio-layout-pane-"))
         (tab-bar-pane (gensym "tab-bar-pane-"))
         (tabbed-layout-name-gensym (gensym "tabbed-layout-"))
         (tabbed-layout-name (or name tabbed-layout-name-gensym)))
    `(let ((,tab-bar-pane (make-clim-stream-pane
                                   :default-view +tab-bar-view+
                                   :display-time :command-loop
                                   :scroll-bars nil
                                   :borders nil
                                   :height 22
                                   :display-function
                                   (lambda (frame pane)
                                     (declare (ignore frame))
                                     (stream-increment-cursor-position pane 0 3)
                                     (draw-line* pane 0 17 (slot-value pane 'climi::current-width) 17 :ink +black+)
                                     (mapcar (lambda (tabbed-pane)
                                                 (with-output-as-presentation (pane (tabbed-pane-pane tabbed-pane)
                                                                                    (or (tabbed-pane-ptype tabbed-pane)
                                                                                        ',default-ptype))
                                                   (present tabbed-pane 'tabbed-pane :stream pane)))
                                             (tabbed-panes-of-tabbed-layout (sheet-parent
                                                                             (sheet-parent
                                                                              (or (climi::pane-border pane) pane))))))))
           (,radio-layout-pane (make-pane 'radio-layout-pane
                                          :contents (list ,@(mapcar #'second body)))))
       (make-pane 'tabbed-layout-pane
                  :name ',tabbed-layout-name
                  :tabbed-panes (list ,@(mapcar (lambda (tab-spec)
                                                  `(apply #'make-tabbed-pane-from-list (list ,@tab-spec)))
                                                body))
                  :radio-layout-pane ,radio-layout-pane
                  :tab-bar-pane ,tab-bar-pane
                  :contents (list ,tab-bar-pane
                                  `(+fill+ ,,radio-layout-pane))))))



(defmethod add-pane ((pane tabbed-pane) (parent tabbed-layout-pane) &optional enabled)
  (add-pane (tabbed-pane-pane pane) (radio-layout-pane-of-tabbed-layout parent) enabled)
  (push pane (tabbed-panes-of-tabbed-layout parent)))

(defmethod remove-pane ((pane tabbed-pane) (parent tabbed-layout-pane))
  (remove-pane (tabbed-pane-pane pane) (radio-layout-pane-of-tabbed-layout parent))
  (setf (tabbed-panes-of-tabbed-layout parent)
        (delete pane (tabbed-panes-of-tabbed-layout parent)
                :test #'equal)))

(defmethod remove-pane ((pane sheet) (parent tabbed-layout-pane))
  (remove-pane (find-in-tabbed-panes-list pane parent) parent))

(defmethod remove-pane ((pane string) (parent tabbed-layout-pane))
  (remove-pane (find-in-tabbed-panes-list pane parent) parent))

(defmethod remove-pane ((pane tabbed-pane) parent)
  (declare (ignore parent))
  (remove-pane pane (sheet-parent (sheet-parent (tabbed-pane-pane pane)))))

(defmethod remove-pane ((pane sheet) (parent (eql 'tabbed-layout-pane)))
  (let ((parent (sheet-parent (sheet-parent pane))))
    (remove-pane (find-in-tabbed-panes-list pane parent) parent)))
  

;;; example code

(define-application-frame ozo ()
  ()
  (:panes
   (editor :text-editor :value "Text Editor")
   (foo :text-editor :value "Foo")
   (bar (make-pane 'push-button :label "Bar" :name "bar pane"))
   (baz (make-pane 'push-button :label "Baz"))
   (io :interactor :height 20 :width 600)
   (pointer-doc :pointer-documentation))
  (:layouts
   (default
      (vertically ()
        (with-tabbed-layout (pane :name tab-pane) ;; XXX no quote for :name
           ("Text Editor" editor)
           ("foo pane(i)" foo 'integer)
           ("bar pane" bar)
           ("baz pane" baz))
        io
        pointer-doc))))

(defun ozo ()
  (run-frame-top-level (make-application-frame 'ozo)))

(define-presentation-to-command-translator remove-pane
    (pane com-remove-tabbed-pane ozo
                 :gesture :describe
                 :documentation "remove this pane"
                 :pointer-documentation "remove this pane")
    (object)
  (list object))


(define-ozo-command (com-remove-pane-by-title :name t)
    ((title 'string :prompt "Pane title"))
  (remove-pane title (find-pane-named *application-frame* 'tab-pane)))

(define-ozo-command (com-remove-pane-by-pane :name t)
    ((sheet 'sheet :prompt "Sheet"))
  (remove-pane sheet (find-pane-named *application-frame* 'tab-pane)))


(define-ozo-command (com-add-brr-pane :name t)
    ()
  (let ((fm (frame-manager *application-frame*)))
    (with-look-and-feel-realization (fm *application-frame*)
      (add-pane (make-tabbed-pane-from-list "Brr"
                                            (make-pane 'text-editor-pane :name 'brr :value "Brr"))
                (find-pane-named *application-frame* 'tab-pane) t))))

(define-ozo-command (com-remove-brr-pane-by-pane :name t)
    ()
  (let ((pane (find-pane-named *application-frame* 'brr)))
    ;; (remove-pane pane (sheet-parent (sheet-parent pane)))
    (remove-pane pane 'tabbed-layout-pane)))

(define-ozo-command (com-remove-brr-pane-by-title :name t)
    ()
  (remove-pane "Brr" (find-pane-named *application-frame* 'tab-pane)))

;;; This does not work class instances are not equal (even not under EQUALP).
;;; It could be made working by replacing the class TABBED-PANE by a STRUCTure, though.
;;;
;;; (define-ozo-command (com-remove-pane :name t)
;;;     ()
;;;   (let ((pane (find-pane-named *application-frame* 'brr)))
;;;     (remove-pane (make-tabbed-pane-from-list "Brr" pane)
;;;                  (sheet-parent (sheet-parent pane)))))
